unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, Buttons, mdBUTTONS, CheckLst, ExtCtrls;

type
  TComp=packed record
    Reference:String;
    Partname:String;
    Package:String;
    Value:String;
  end;

  TPart=packed record
    Reference:String;
    Partname:String;
    Package:String;
    RestOfLine:String;
  end;

  TMainForm = class(TForm)
    PartlistLabel: TLabel;
    PartlistSaveXPBitBtn: TXPBitBtn;
    FileOpenDialog: TOpenDialog;
    NetlistEdit: TEdit;
    NetlistLabel: TLabel;
    PartlistEdit: TEdit;
    PartlistCheckListBox: TCheckListBox;
    CompareXPBitBtn: TXPBitBtn;
    InfoTimer: TTimer;
    NetlistInfoLabel: TLabel;
    PartlistInfoLabel: TLabel;
    ReplaceXPBitBtn: TXPBitBtn;
    UncheckAllXPBitBtn: TXPBitBtn;
    procedure NetlistEditClick(Sender: TObject);
    procedure PartlistEditClick(Sender: TObject);
    procedure PartlistCheckListBoxDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; AState: TOwnerDrawState);
    procedure PartlistCheckListBoxClickCheck(Sender: TObject);
    procedure CompareXPBitBtnClick(Sender: TObject);
    procedure InfoTimerTimer(Sender: TObject);
    procedure ReplaceXPBitBtnClick(Sender: TObject);
    procedure UncheckAllXPBitBtnClick(Sender: TObject);
    procedure PartlistSaveXPBitBtnClick(Sender: TObject);
  private
    NetlistCompArr:Array of TComp;
    PartlistCompArr:Array of TPart;
    procedure LoadNetlistCompArr(AFileName:String);
    procedure LoadPartlistCompArr(AFileName:String);
  public
  end;

var
  MainForm: TMainForm;

{================================================================================================}
implementation
{$R *.DFM}

procedure TMainForm.NetlistEditClick(Sender: TObject);

begin
  with FileOpenDialog do
  begin
    DefaultExt:='PNL';
    Filter:='PNL Files (*.PNL)|*.PNL|All Files (*.*)|*.*';
    Title:='Open netlist';
    FileName:=NetlistEdit.Text;

    if Execute then
    begin
      NetlistEdit.Text:=FileName;

      LoadNetlistCompArr(NetlistEdit.Text);
    end;
  end;
end;


procedure TMainForm.PartlistEditClick(Sender: TObject);

var
  x:Integer;

begin
  with FileOpenDialog do
  begin
    DefaultExt:='CSV';
    Filter:='CSV Files (*.CSV)|*.CSV|All Files (*.*)|*.*';
    Title:='Open part list';
    FileName:=PartlistEdit.Text;

    if Execute then
    begin
      PartlistEdit.Text:=FileName;

      with PartlistCheckListBox,Items do
      begin
        LoadPartlistCompArr(PartlistEdit.Text);

        Clear;
        LoadFromFile(PartlistEdit.Text);
        if (Count>0) then Delete(0);
        ItemIndex:=-1;
        for x:=0 to Count-1 do Checked[x]:=false;
      end;
    end;
  end;
end;


procedure TMainForm.PartlistCheckListBoxClickCheck(Sender: TObject);

begin
  PartlistCheckListBox.Repaint;
end;


procedure TMainForm.PartlistCheckListBoxDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; AState: TOwnerDrawState);

const
  ColumnWidth=80;

var
  s:String;
  x:Integer;

begin
  with TCheckListBox(Control),Canvas do
  begin
    if Checked[Index] then
      if (odSelected in AState) then Brush.Color:=clMaroon else Brush.Color:=clRed;

    FillRect(Rect);
    Rect.Left:=Rect.Left;

    s:=Items[Index];
    Rect.Right:=Rect.Left+ColumnWidth;
    repeat
      x:=Pos(#$09,s);
      TextRect(Rect,Rect.Left,Rect.Top,Copy(s,1,x-1));
      s:=Copy(s,x+1,Length(s));
      OffsetRect(Rect,ColumnWidth,0);
    until (x=0);
  end;
end;


procedure TMainForm.InfoTimerTimer(Sender: TObject);

var
  x,i:Integer;

begin
  InfoTimer.Interval:=500;

  NetlistInfoLabel.Caption:=Format('Items: %d',[Length(NetlistCompArr)]);

  i:=0;
  with PartlistCheckListBox, Items do for x:=0 to Count-1 do
    if Checked[x] then Inc(i);

  PartlistInfoLabel.Caption:=Format('Items: %d, Checked: %d',[Length(PartlistCompArr), i]);
end;


{------------------------------------------------------------------------------------------------}

procedure TMainForm.LoadNetlistCompArr(AFileName:String);

var
  starti,endi,x:Integer;
  i:Integer;
  s:String;

begin
  NetlistCompArr:=nil;

  with TStringList.Create do
  try
    LoadFromFile(AFileName);
    starti:=IndexOf('Components (')+1;
    endi:=IndexOf('Netlist (')-2;

    if (starti>0) then for x:=starti to endi do
    begin
      SetLength(NetlistCompArr,Length(NetlistCompArr)+1);
      with NetlistCompArr[High(NetlistCompArr)] do
      begin
        s:=Strings[x];
        s:=Copy(s,Pos('("',s)+2,Length(s));

        i:=Pos('"',s);
        Reference:=Copy(s,1,i-1);
        s:=Copy(s,i+1,Length(s));
        s:=Copy(s,Pos('"',s)+1,Length(s));

        i:=Pos('"',s);
        Partname:=Copy(s,1,i-1);
        s:=Copy(s,i+1,Length(s));
        s:=Copy(s,Pos('"',s)+1,Length(s));

        i:=Pos('"',s);
        Package:=Copy(s,1,i-1);
        s:=Copy(s,i+1,Length(s));
        s:=Copy(s,Pos('"',s)+1,Length(s));

        i:=Pos('"',s);
        Value:=Copy(s,1,i-1);
      end;
    end;
  finally
    Free;
  end;
end;

{--------}

procedure TMainForm.LoadPartlistCompArr(AFileName:String);

var
  starti,endi,x:Integer;
  i:Integer;
  s:String;

begin
  PartlistCompArr:=nil;

  with TStringList.Create do
  try
    LoadFromFile(AFileName);
    starti:=1;
    endi:=Count-1;

    if (starti>0) then for x:=starti to endi do
    begin
      SetLength(PartlistCompArr,Length(PartlistCompArr)+1);
      with PartlistCompArr[High(PartlistCompArr)] do
      begin
        s:=Strings[x];

        i:=Pos(#$09,s);
        Reference:=Copy(s,1,i-1);
        s:=Copy(s,i+1,Length(s));

        i:=Pos(#$09,s);
        Partname:=Copy(s,1,i-1);
        s:=Copy(s,i+1,Length(s));

        i:=Pos(#$09,s);
        Package:=Copy(s,1,i-1);

        RestOfLine:=Copy(s,i,Length(s));
      end;
    end;
  finally
    Free;
  end;
end;

{================================================================================================}

procedure TMainForm.CompareXPBitBtnClick(Sender: TObject);

var
  x,y:Integer;
  found:Boolean;

begin
  for x:=0 to High(PartlistCompArr) do
  begin
    y:=High(NetlistCompArr);
    found:=false;
    while (y>=0) and (not found) do
    begin
      found:=(PartlistCompArr[x].Reference=NetlistCompArr[y].Reference);
      Dec(y);
    end;

    PartlistCheckListBox.Checked[x]:=not found;
  end;

  PartlistCheckListBox.Repaint;
end;


procedure TMainForm.ReplaceXPBitBtnClick(Sender: TObject);

var
  x,y:Integer;
  found:Boolean;
  yestoall:Boolean;
  stop:Boolean;
  replaced:Integer;

function CanReplace(const AWhat:String; const ABy:String):Boolean;
begin
  Result:=(AWhat<>ABy);
  if not(Result) then Exit;

  Result:=yestoall;

  if not(Result) then
  case MessageDlg('Replace "'+AWhat+'"'+#$0A#$0D+'by "'+ABy+'"?',mtConfirmation,[mbYes,mbYesToAll,mbNo],0) of
    mrYes:Result:=true;
    mrYesToAll:begin
                 Result:=true;
                 yestoall:=true;
               end;
    mrCancel:stop:=true;
  end;
end;

begin
  yestoall:=false;
  stop:=false;
  replaced:=0;
  for x:=0 to High(PartlistCompArr) do
  begin
    y:=High(NetlistCompArr);
    found:=false;
    while (y>=0) and (not found) do
    begin
      found:=(PartlistCompArr[x].Reference=NetlistCompArr[y].Reference);
      Dec(y);
    end;

    if found then
    begin
      PartlistCheckListBox.ItemIndex:=x;

      if (NetlistCompArr[y+1].Value<>' ') and CanReplace(PartlistCompArr[x].Partname,NetlistCompArr[y+1].Value)
      then with PartlistCompArr[x] do
      begin
        Partname:=NetlistCompArr[y+1].Value;
        PartlistCheckListBox.Items[x]:=
          Reference +#$09+ Partname +#$09+ Package + RestOfLine;
        Inc(replaced);
      end;
    end;

    if stop then Break;
  end;

  MessageDlg('Replaced '+IntToStr(replaced)+' items.',mtInformation,[mbOK],0);
end;


procedure TMainForm.UncheckAllXPBitBtnClick(Sender: TObject);

var
  x:Integer;

begin
  with PartlistCheckListBox, Items do
    for x:=0 to Count-1 do Checked[x]:=false;

  PartlistCheckListBox.Repaint;
end;

{================================================================================================}

procedure TMainForm.PartlistSaveXPBitBtnClick(Sender: TObject);

var
  x:Integer;
  
begin
  with TStringList.Create do
  try
    Add('Modified Part List');
    for x:=0 to PartlistCheckListBox.Items.Count-1 do
      if not PartlistCheckListBox.Checked[x] then with PartlistCompArr[x] do
        Add(Reference +#$09+ Partname +#$09+ Package + RestOfLine);

    SaveToFile(PartlistEdit.Text+'.mod');
    MessageDlg('Stored '+IntToStr(Count-1)+' items to file'+#$0A#$0D+PartlistEdit.Text+'.mod .',mtInformation,[mbOK],0);
  finally
    Free;
  end;
end;

{================================================================================================}


end.

